package com.nisum.productreader.service;

import com.nisum.productreader.model.Product;
import com.opencsv.CSVReader;
import com.opencsv.exceptions.CsvException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;

import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.List;

@Service
public class ProductService {

    private static final Logger logger = LoggerFactory.getLogger(ProductService.class);

    public Flux<Product> readProductFromCsv() {
        logger.info("Starting to read the CSV file from classpath");

        return Flux.create(sink -> {
            try {
                // Load CSV file from classpath
                ClassPathResource resource = new ClassPathResource("product.csv");
                CSVReader csvReader = new CSVReader(new InputStreamReader(resource.getInputStream()));

                List<String[]> rows = csvReader.readAll();
                logger.info("Successfully read {} rows from the CSV file", rows.size() - 1);

                rows.stream().skip(1).forEach(row -> {
                    Product product = new Product();
                    product.setProductId(Integer.parseInt(row[0]));
                    product.setProductName(row[1]);
                    product.setProductCategory(row[2]);

                    // Convert list prices into List<Double>
                    List<Double> prices = Arrays.asList(row[3].split(","))
                            .stream().map(String::trim)
                            .map(Double::parseDouble).toList();
                    product.setPriceList(prices);

                    product.setColor(row[4]);
                    product.setRam(row[5]);
                    product.setModel(row[6]);

                    logger.info("Parsed product: {}", product); // Log product details at debug level

                    sink.next(product); // Emit product
                });
                sink.complete(); // Complete the flux
                logger.info("Completed reading and processing the CSV file.");
            } catch (Exception e) {
                logger.error("Error reading CSV file: {}", e.getMessage(), e);
                sink.error(e); // Error signal if exception occurs
            }
        });
    }
}

